{$I-}

uses dos;

const
  _tab = #9;

var
  source_file_name, destination_file_name, command_string: string;
  source_file: file;
  destination_file: text;
  command: integer;
  answer: char;
  really: boolean;

procedure Display_help;
begin
  writeln('FILE-INC by Zed/reMal.');
  writeln('This utility converts a data file into an include-style one.');
  writeln('Parameters:');
  writeln('FILE-INC.EXE <command> <source file> <destination file>');
  writeln;
  writeln('Commands are:');
  writeln('-ab: Assembly-style output file of bytes.');
  writeln('-aw: Assembly-style output file of words.');
  writeln('-ad: Assembly-style output file of doublewords.');
  writeln('-pb: Turbo Pascal-style output file of bytes.');
  writeln('-pw: Turbo Pascal-style output file of words.');
  writeln('-pd: Turbo Pascal-style output file of doublewords.');
end;

function Convert_file(var source_file: file; var destination_file: text;
  command: integer): integer;

var
  read_buffer: longint;
  element_size, field_length, i, error: integer;
  temp_str, line_str, line_begin, line_end, directive_str: string;

begin
  error:= 0;
  case command and $0f of
    1:
    begin
      element_size:= 1;
      field_length:= 3;
      directive_str:= 'db';
    end;
    2:
    begin
      element_size:= 2;
      field_length:= 5;
      directive_str:= 'dw';
    end;
    3:
    begin
      element_size:= 4;
      field_length:= 10;
      directive_str:= 'dd';
    end;
    else error:= 1;
  end;
  case command shr 4 of
    0:
    begin
      line_begin:= _tab+directive_str+_tab;
      line_end:= '';
    end;
    1:
    begin
      line_begin:= '  ';
      line_end:= ',';
    end;
    else error:= 1;
  end;

  if error=0 then
  begin
    read_buffer:= 0;
    while not(eof(source_file)) do
    begin
      i:= 0;
      line_str:= '';
      while not(eof(source_file)) and (i<8) do
      begin
        read_buffer:= 0;
        blockread(source_file, read_buffer, element_size);
        str(read_buffer: field_length, temp_str);
        line_str:= line_str+', '+temp_str;
        inc(i);
      end;
      if i>0 then
      begin
        line_str:= line_begin+copy(line_str, 3, length(line_str)-2);
        if not(eof(source_file)) then line_str:= line_str+line_end;
        writeln(destination_file, line_str);
      end;
    end;
  end;
  Convert_file:= error;
end;

begin
  case paramcount of
    3:
    begin
      command_string:= paramstr(1);
      source_file_name:= fexpand(paramstr(2));
      destination_file_name:= fexpand(paramstr(3));
      command:= 0;
      if command_string='-ab' then command:= $01;
      if command_string='-aw' then command:= $02;
      if command_string='-ad' then command:= $03;
      if command_string='-pb' then command:= $11;
      if command_string='-pw' then command:= $12;
      if command_string='-pd' then command:= $13;
      if command<>0 then
      begin
        if source_file_name<>destination_file_name then
        begin
          assign(source_file, source_file_name);
          assign(destination_file, destination_file_name);
          reset(source_file, 1);
          if ioresult=0 then
          begin
            reset(destination_file);
            if ioresult=0 then
            begin
              write('WARNING: Output file already exists. ');
              repeat
                write('Overwrite? [Y]es, [N]o: ');
                readln(answer);
              until (upcase(answer)='Y') or (upcase(answer)='N');
              really:= (upcase(answer)='Y');
            end
            else really:= true;
            if really then
            begin
              rewrite(destination_file);
              if Convert_file(source_file, destination_file, command)<>0 then
                writeln('ERROR: Conversion failed!');
            end
            else writeln('ERROR: Operation cancelled.');
            close(destination_file);
          end
          else writeln('ERROR: File not found.');
          close(source_file);
        end
        else writeln('ERROR: Source and destination files must be different.');
      end
      else writeln('ERROR: Invalid command.');
    end;
    else Display_help;
  end;
end.
