uses dos;

const
  tab = #9;

var
  lbm_file_name, pal_file_name: string;
  lbm_dir, pal_dir: dirstr;
  lbm_name, pal_name: namestr;
  lbm_ext, pal_ext: extstr;
  dos_error, error_code: word;
  search_rec: searchrec;
  first_color, num_colors, i: word;

procedure Display_help;
begin
  writeln('Palette extractor utility by Zed/reMal.');
  writeln('This program can copy the palette from an LBM or BBM file to a text file');
  writeln('that can be INCLUDEd in assembly source.');
  writeln('Parameters: LBMPAL <lbm-file name> <first color> <number of colors>');
  writeln('LBMPAL <lbm-file name (Wildcards allowed.)> <first color> <number of colors>');
end;

procedure Save_LBM_pal(lbm_file_name, pal_file_name: string;
  first_color, num_colors: word);
var
  lbm_file: file;
  pal_file: text;
  palette: array[0..255, 1..3] of byte;
  planes_num, max_color, i: word;
  red_str, green_str, blue_str: string;

begin
  assign(lbm_file, lbm_file_name);
  reset(lbm_file, 1);
  seek(lbm_file, $1c);
  planes_num:= 0;
  blockread(lbm_file, planes_num, 1);
  case planes_num of
    1: max_color:= 2;
    2: max_color:= 4;
    4: max_color:= 16;
    8: max_color:= 256;
  else max_color:= 0;
  end;

  if (max_color>0) and (first_color+num_colors<=max_color) then
  begin
    writeln('Extracting palette from '+lbm_file_name+' to '+pal_file_name);
    seek(lbm_file, $30+3*first_color);
    blockread(lbm_file, palette[first_color], num_colors*3);
    assign(pal_file, pal_file_name);
    rewrite(pal_file);
    writeln(pal_file, '; first_color'+tab+tab+'equ'+tab, first_color);
    writeln(pal_file, '; number_of_colors'+tab+'equ'+tab, num_colors);
    writeln(pal_file);
    for i:= first_color to first_color+num_colors-1 do
    begin
      str(palette[i, 1] div 4: 2, red_str);
      str(palette[i, 2] div 4: 2, green_str);
      str(palette[i, 3] div 4: 2, blue_str);
      writeln(pal_file, tab+'db'+tab+
        red_str+', '+green_str+', '+blue_str+tab+'; ', i);
    end;
    close(pal_file);
    end
  else
    writeln('Cannot extract a palette of that size from '+lbm_file_name);

  close(lbm_file);
end;

begin
  if paramcount=3 then
  begin
    val(paramstr(2), first_color, error_code);
    if error_code<>0 then
    begin
      Display_help;
      exit;
    end;
    val(paramstr(3), num_colors, error_code);
    if error_code<>0 then
    begin
      Display_help;
      exit;
    end;
    lbm_file_name:= paramstr(1);
    fsplit(lbm_file_name, lbm_dir, lbm_name, lbm_ext);
    for i:= 1 to length(lbm_ext) do lbm_ext[i]:= upcase(lbm_ext[i]);
    if (lbm_ext='.LBM') or (lbm_ext='.BBM') then
    begin
      findfirst(lbm_file_name, anyfile, search_rec);
      if doserror=0 then
      begin
        while doserror=0 do
        begin
          fsplit(search_rec.name, pal_dir, pal_name, pal_ext);
          lbm_file_name:= lbm_dir+search_rec.name;
          pal_file_name:= lbm_dir+pal_name+'.PAL';
          Save_LBM_pal(lbm_file_name, pal_file_name, first_color, num_colors);
          findnext(search_rec);
        end;
      end
     else writeln('Can''t find '+lbm_file_name);
    end
    else writeln(lbm_file_name+' is not an LBM file.');
  end
  else Display_help;
end.
