;ͻ
; Procedure name: Morph_signed_bytes                                    Zed 
;͹
; Aim: Slide an array of signed bytes towards another one in equal steps.    
;Ķ
; Input: DS:SI: Points to the array to reach.                                
;        ES:DI: Points to the array to change.                               
;           CX: Number of array elements.                                    
;           DL: I dunno... (1-127.)                                          
;Ķ
; Output: -                                                                  
;Ķ
; Destroys: Flags                                                            
;Ķ
; Note: First call with the number of steps desired in DL, then decrease DL  
;  to 1 for consecutive calls.                                               
;ͼ
proc	Morph_signed_bytes
	pusha

@@morph_next_byte:
	lodsb
	mov	dh,es:[di]
	sub	al,dh
	cbw
	idiv	dl
	add	al,dh
	stosb
	loop	short @@morph_next_byte

	popa
	ret
endp	Morph_signed_bytes

;ͻ
; Procedure name: Wait_vertical_retrace                                 Zed 
;͹
; Aim: Waits for start of vertical retrace.                                  
;Ķ
; Input: -                                                                   
;Ķ
; Output: -                                                                  
;Ķ
; Destroys: Flags                                                            
;ͼ
proc	Wait_vertical_retrace
	pusha

	mov	dx,3dah
@@wait_retrace_1:
	in	al,dx
	test	al,08h
	jnz	short @@wait_retrace_1
@@wait_retrace_2:
	in	al,dx
	test	al,08h
	jz	short @@wait_retrace_2

	popa
	ret
endp	Wait_vertical_retrace

;ͻ
; Procedure name: Palette_out                                           Zed 
;͹
; Aim: Output palette to VGA card.                                           
;Ķ
; Input: AL: Number of first color to be set.                                
;        CX: Number of colors to be set.                                     
;        DS:SI: Palette in RGB, RGB... format.                               
;Ķ
; Output: -                                                                  
;Ķ
; Destroys: Flags                                                            
;ͼ
proc	Palette_out
	pusha

	mov	dx,3c8h
	out	dx,al
	inc	dx
@@next_byte:
	lodsb
	out	dx,al
	lodsb
	out	dx,al
	lodsb
	out	dx,al
	loop	short @@next_byte

	popa
	ret
endp	Palette_out

;ͻ
; Procedure name: Rotate_point_Z                                        Zed 
;͹
; Aim: Computes the coordinates of a point after being rotated on the Z      
;      axis.                                                                 
;Ķ
; Input: AX: Original X coordinate of point.                                 
;        BX: Original Y coordinate.                                          
;        DX: Angle of rotation around the Z axis. (<type(sine)> aligned.)    
;        DS:sine: Array of sine values.                                      
;        DS:cosine: Array of cosine values.                                  
;Ķ
; Output: AX: Rotated X coordinate.                                          
;         BX: Rotated Y coordinate.                                          
;Ķ
; Destroys: Flags, AX, BX                                                    
;ͼ
proc	Rotate_point_Z
	push	cx
	push	dx
	push	si
	push	di
	push	bp

	mov	si,dx
	mov	di,ax
	mov	bp,bx

; Compute rotated Y coordinate.
	imul	sine[si]
	mov	bx,ax
	mov	cx,dx
	mov	ax,bp
	imul	cosine[si]
	sub	ax,bx
	sbb	dx,cx
	idiv	sine_multiplier
	push	ax

; Compute rotated X coordinate.
	mov	ax,di
	imul	cosine[si]
	mov	bx,ax
	mov	cx,dx
	mov	ax,bp
	imul	sine[si]
	add	ax,bx
	adc	dx,cx
	idiv	sine_multiplier
	pop	bx

	pop	bp
	pop	di
	pop	si
	pop	dx
	pop	cx
	ret
endp	Rotate_point_Z

;ͻ
; Procedure name: Rotate_matrix                                         Zed 
;͹
; Aim: Computes the rotation matrix for the given angles.                    
;Ķ
; Input: AX: Angle of rotation around the X axis. (Type-aligned.)            
;        BX: Same for the Y axis.                 (Type-aligned.)            
;        CX: Same for the Z axis.                 (Type-aligned.)            
;        DS:sine: Array of sine values.                                      
;        DS:cosine: Array of cosine values.                                  
;Ķ
; Output: DS:rotation_matrix: Values are stored here.                        
;Ķ
; Destroys: Flags                                                            
;ͼ
proc	Rotate_matrix
	pusha

	mov	di,cx
	mov	si,bx
	mov	bx,ax
	mov	bp,_sine_multiplier

	mov	ax,cosine[si]
	imul	cosine[di]
	idiv	bp
	mov	rotation_matrix[0*6+0*2],ax

	mov	ax,cosine[si]
	imul	sine[di]
	idiv	bp
	neg	ax
	mov	rotation_matrix[0*6+1*2],ax

	mov	ax,sine[si]
	mov	rotation_matrix[0*6+2*2],ax

	mov	ax,sine[bx]
	imul	sine[si]
	idiv	bp
	imul	cosine[di]
	idiv	bp
	mov	cx,ax
	mov	ax,cosine[bx]
	imul	sine[di]
	idiv	bp
	add	ax,cx
	mov	rotation_matrix[1*6+0*2],ax

	mov	ax,sine[bx]
	imul	sine[si]
	idiv	bp
	imul	sine[di]
	idiv	bp
	mov	cx,ax
	mov	ax,cosine[bx]
	imul	cosine[di]
	idiv	bp
	sub	ax,cx
	mov	rotation_matrix[1*6+1*2],ax

	mov	ax,sine[bx]
	imul	cosine[si]
	idiv	bp
	neg	ax
	mov	rotation_matrix[1*6+2*2],ax

	mov	ax,cosine[bx]
	imul	sine[si]
	idiv	bp
	imul	cosine[di]
	idiv	bp
	mov	cx,ax
	mov	ax,sine[bx]
	imul	sine[di]
	idiv	bp
	sub	ax,cx
	mov	rotation_matrix[2*6+0*2],ax

	mov	ax,sine[bx]
	imul	cosine[di]
	idiv	bp
	mov	cx,ax
	mov	ax,cosine[bx]
	imul	sine[si]
	idiv	bp
	imul	sine[di]
	idiv	bp
	add	ax,cx
	mov	rotation_matrix[2*6+1*2],ax

	mov	ax,cosine[bx]
	imul	cosine[si]
	idiv	bp
	mov	rotation_matrix[2*6+2*2],ax

	popa
	ret
endp	Rotate_matrix

;ͻ
; Procedure name: Rotate_point_matrix                                   Zed 
;͹
; Aim: Computes the coordinates of a point after being rotated by the angles 
;      given.                                                                
;Ķ
; Input: AX: Original X coordinate of point.                                 
;        BX: Original Y coordinate.                                          
;        CX: Original Z coordinate.                                          
;        DS:rotation_matrix: Rotation matrix computed by Rotate_matrix.      
;Ķ
; Output: AX: Rotated X coordinate.                                          
;         BX: Rotated Y coordinate.                                          
;         CX: Rotated Z coordinate.                                          
;Ķ
; Destroys: Flags, AX, BX, CX                                                
;ͼ
proc	Rotate_point_matrix
	push	dx
	push	si
	push	di
	push	bp

	mov	si,ax
	mov	di,bx
	mov	bp,cx

; Compute rotation around Z axis.
	mov	ax,si
	imul	rotation_matrix[0*6+2*2]
	mov	bx,ax
	mov	cx,dx
	mov	ax,di
	imul	rotation_matrix[1*6+2*2]
	add	bx,ax
	adc	cx,dx
	mov	ax,bp
	imul	rotation_matrix[2*6+2*2]
	add	ax,bx
	adc	dx,cx
	idiv	sine_multiplier
	push	ax

; Compute rotation around Y axis.
	mov	ax,si
	imul	rotation_matrix[0*6+1*2]
	mov	bx,ax
	mov	cx,dx
	mov	ax,di
	imul	rotation_matrix[1*6+1*2]
	add	bx,ax
	adc	cx,dx
	mov	ax,bp
	imul	rotation_matrix[2*6+1*2]
	add	ax,bx
	adc	dx,cx
	idiv	sine_multiplier
	push	ax

; Compute rotation around X axis.
	mov	ax,si
	imul	rotation_matrix[0*6+0*2]
	mov	bx,ax
	mov	cx,dx
	mov	ax,di
	imul	rotation_matrix[1*6+0*2]
	add	bx,ax
	adc	cx,dx
	mov	ax,bp
	imul	rotation_matrix[2*6+0*2]
	add	ax,bx
	adc	dx,cx
	idiv	sine_multiplier

	pop	bx			; Pop rotated Y and Z coordinates.
	pop	cx

	pop	bp
	pop	di
	pop	si
	pop	dx
	ret
endp	Rotate_point_matrix

;ͻ
; Procedure name: Fill_polygon                                          Zed 
;͹
; Aim: Fill a convex polygon.                                                
;Ķ
; Input: AL: Color of the polygon.                                           
;        DX: Number of edges.                                                
;        DS:SI: Points to the polygon.                                       
;        ES: Segment of the video RAM.                                       
;        DS:calc_polygon_edge_offset: Offset of the edge-drawing procedure.  
;        DS:fill_polygon_lines_offset: Offset of horizontal line-drawer proc.
;Ķ
; Output: -                                                                  
;Ķ
; Destroys: Flags                                                            
;ͼ
proc	Fill_polygon
	pusha

	mov	fill_polygon_color,al
	dec	dx

	mov	cx,dx
	mov	bx,si
	mov	di,[bx]
	mov	ax,word ptr transed_Y[di]	; Top line.
	mov	bp,ax				; Bottom line.
@@top_bottom:
	add	bx,2
	mov	di,[bx]
	mov	di,word ptr transed_Y[di]
	cmp	ax,di
	jle	short @@not_new_top
	mov	ax,di
@@not_new_top:
	cmp	bp,di
	jge	short @@not_new_bottom
	mov	bp,di
@@not_new_bottom:
	loop	short @@top_bottom
	add	ax,ax
	mov	fill_first_line,ax
	add	bp,bp
	mov	fill_last_line,bp

	push	si
	mov	cx,dx
@@next_polygon_line:
	push	cx
	mov	di,[si]
	mov	ax,word ptr transed_X[di]
	mov	cx,word ptr transed_Y[di]
	mov	di,[si+2]
	mov	bx,word ptr transed_X[di]
	mov	dx,word ptr transed_Y[di]
	push	si
	call	word ptr [calc_polygon_edge_offset]
	pop	si
	add	si,2
	pop	cx
	loop	short @@next_polygon_line

	mov	di,[si]
	mov	ax,word ptr transed_X[di]
	mov	cx,word ptr transed_Y[di]
	pop	si
	mov	di,[si]
	mov	bx,word ptr transed_X[di]
	mov	dx,word ptr transed_Y[di]
	call	word ptr [calc_polygon_edge_offset]

	call	word ptr [fill_polygon_lines_offset]

	popa
	ret
endp	Fill_polygon

;ͻ
; Procedure name: Calc_polygon_edge                                     Zed 
;͹
; Aim: Calculate X coordinates of one edge of a polygon.                     
;Ķ
; Input: AX: X1 coordinate of the polygon's edge.                            
;        BX: X2        --- " ---                                             
;        CX: Y1        --- " ---                                             
;        DX: Y2.       --- " ---                                             
;Ķ
; Output: DS:fill_line_X_1 or DS:fill_line_X_2: X coordinates of the line.   
;Ķ
; Destroys: Flags, AX, BX, CX, DX, SI, DI, BP                                
;ͼ
proc	Calc_polygon_edge
	mov	di,offset fill_line_X_1	; Suppose the line goes down.
	cmp	cx,dx			; Is Y1<=Y2 ?
	jle	short @@Y_OK
	mov	di,offset fill_line_X_2	; Goes up.

@@Y_OK:
	cmp	ax,bx			; Is X1<=X2 ?
	jle	short @@X_ok		; Yes.
	xchg	ax,bx			; Else swap coordinates.
	xchg	cx,dx

@@X_ok:
	mov	bp,2			; Suppose the line goes down.
	sub	dx,cx			; DX:= Y difference.
	jge	short @@goes_down
	neg	bp
	neg	dx

@@goes_down:
	add	di,cx			; Offset in arrays fill_line_X_...
	add	di,cx
	sub	bx,ax			; X difference.
	inc	bx			; Increase both differences to get a
	inc	dx			; smoother line.

	cmp	dx,bx			; Is dY>dX ? (Both are positive.)
	jae	short @@line_vertical

;Ŀ
; At this point registers are as follows:                                    
; AX: X1 coordinate from the caller.                                         
; DI: Offset of the array to put X coordinates in.                           
; BX: X difference. ABS(X2-X1)+1.                                            
; DX: Y difference. ABS(Y2-Y1)+1.                                            
; BP: If the line goes down, then 2, if it goes up the -2.                   
;
@@line_horizontal:
	mov	cx,bx
	mov	si,bx
@@line_horizontal_1:
	mov	[di],ax
	inc	ax
	sub	si,dx
	jg	short @@line_horizontal_2
	add	di,bp
	add	si,bx
@@line_horizontal_2:
	loop	short @@line_horizontal_1
	jmp	short @@line_ready

@@line_vertical:
	mov	cx,dx
	mov	si,dx
@@line_vertical_1:
	mov	[di],ax
	add	di,bp
	sub	si,bx
	jg	short @@line_vertical_2
	inc	ax
	add	si,dx
@@line_vertical_2:
	loop	short @@line_vertical_1

@@line_ready:
	ret
endp	Calc_polygon_edge

;ͻ
; Procedure name: Fill_polygon_lines_mode_0dh                           Zed 
;͹
; Aim: Draw a horizontal line when filling a polygon on a 320x200x16 screen. 
;Ķ
; Input: DS:fill_first_line: Top line.                                       
;        DS:fill_last_line: Bottom line.                                     
;        DS:fill_line_X_1: X coordinates array 2.                            
;        DS:fill_line_X_2: X coordinates array 2.                            
;        DS:fill_polygon_color: Color of the line.                           
;        ES: The video RAM segment.                                          
;Ķ
; Output: -                                                                  
;Ķ
; Destroys: Flags, AX, BX, CX, DX, SI, DI, BP                                
;ͼ
proc	Fill_polygon_lines_mode_0dh
	mov	al,2
	mov	ah,fill_polygon_color
	mov	dx,3c4h
	out	dx,ax

	mov	si,fill_first_line
@@fill_next_line:
	mov	bx,fill_line_X_1[si]
	mov	cx,fill_line_X_2[si]

	cmp	bx,cx			; Is X1<=X2 ?
	jle	short @@X_ok		; Yes.
	xchg	bx,cx			; Else swap coordinates.

@@X_ok:
	mov	bp,cx
	mov	di,bx
	shr	di,3
	shr	cx,3
	sub	cx,di
	mov	ax,screen_Y_offsets[si]
	add	di,ax

	and	bx,7
	mov	al,fill_bitplanes[bx]
	xor	al,11111111b
	dec	cx
	js	short @@same_byte
	stosb

	jcxz	short @@no_whole_byte
	mov	ax,0ffffh
	test	di,1			; Align to 16-bit boundary.
	jz	short @@no_first_byte
	stosb
	dec	cx
@@no_first_byte:
	shr	cx,1
rep	stosw
	jnc	short @@no_whole_byte
	stosb

@@no_whole_byte:
	mov	bx,bp
	and	bx,7
	mov	al,fill_bitplanes[bx]
	jmp	short @@final_byte

@@same_byte:
	mov	bx,bp
	and	bx,7
	and	al,fill_bitplanes[bx]
@@final_byte:
	mov	es:[di],al

@@exit:
	add	si,2
	cmp	si,fill_last_line
	jbe	short @@fill_next_line

	ret
endp	Fill_polygon_lines_mode_0dh
