{$I-}

uses dos;

const
  _buffer_size = 65520;
  _max_noc = 9;

type
  buffer_type = array[0.._buffer_size-1] of byte;

var
  i, error: integer;
  source_file_name, destination_file_name: string;
  source_file, destination_file: file;
  answer: char;
  really: boolean;

procedure Write_help;
begin
  writeln('S3M-RAT converter by Zed/reMal.');
  writeln('This program converts an AdLib S3M song to RAT (Remal Adlib Tune) format.');
  writeln('');
  writeln('Parameters: S3M-RAT.EXE <source> <destination>');
  writeln('Maximum length of source file is 65520 bytes. (Sorry for this...)');
  writeln('');
end;

procedure Convert(var in_file, out_file: file);
var
  noi, noo, nop: word; {Number Of Instrument|Orders|Patterns.}
  noc: integer;        {Number Of Channels.}
  i, p, r, o: word;
  pack_info: byte;
  in_buffer_ptr: ^buffer_type;
  out_buffer_ptr: ^buffer_type;

begin
  new(in_buffer_ptr);
  new(out_buffer_ptr);
{Read S3M and clear RAT.}
  blockread(in_file, in_buffer_ptr^, _buffer_size);
  fillchar(out_buffer_ptr^, _buffer_size, 0);
{Write ID bytes.}
  out_buffer_ptr^[$00]:= ord('R');
  out_buffer_ptr^[$01]:= ord('A');
  out_buffer_ptr^[$02]:= ord('T');
  out_buffer_ptr^[$03]:= $10;           {Kinda version ID.}
{Copy song title.}
  move(in_buffer_ptr^[$00], out_buffer_ptr^[$04], 28);
{Number of orders is the index of first $FF pattern number in the order list.}
  noo:= 0;
  while in_buffer_ptr^[$60+noo]<>$ff do inc(noo);
  out_buffer_ptr^[$26]:= noo mod 256;
  out_buffer_ptr^[$27]:= noo div 256;
{First order and order loop position - unsupported in S3M.}
  out_buffer_ptr^[$2c]:= 0 mod 256;
  out_buffer_ptr^[$2d]:= 0 div 256;
  out_buffer_ptr^[$2e]:= 0 mod 256;
  out_buffer_ptr^[$2f]:= 0 div 256;
{Copy number of instruments.}
  noi:= in_buffer_ptr^[$23]*256+in_buffer_ptr^[$22];
  out_buffer_ptr^[$28]:= noi mod 256;
  out_buffer_ptr^[$29]:= noi div 256;
{Find maximum pattern number in orderlist.}
  nop:= 0;
  for i:= 0 to noo-1 do
    if in_buffer_ptr^[$60+i]>nop then nop:= in_buffer_ptr^[$60+i];
  inc(nop);
  out_buffer_ptr^[$2a]:= nop mod 256;
  out_buffer_ptr^[$2b]:= nop div 256;
{Initial global volume, speed and BPM.}
  out_buffer_ptr^[$30]:= in_buffer_ptr^[$30];   {Global volume.}
  out_buffer_ptr^[$31]:= in_buffer_ptr^[$31];   {Speed.}
  out_buffer_ptr^[$32]:= in_buffer_ptr^[$32];   {BPM.}
{Copy order list.}
  move(in_buffer_ptr^[$60], out_buffer_ptr^[$40], noo);

{Copy instruments.}
  for i:= 0 to noi do
  begin
    p:= 16*(in_buffer_ptr^[$60+in_buffer_ptr^[$20]+2*i]+
      in_buffer_ptr^[$60+in_buffer_ptr^[$20]+2*i+1]*256); {Pointer to instrument.}
    out_buffer_ptr^[$140+i*$14+$13]:=
      in_buffer_ptr^[p+$00]; {Type.}
    move(in_buffer_ptr^[p+$20],
      out_buffer_ptr^[$140+i*$14+$00], 4); {C4Freq.}
    move(in_buffer_ptr^[p+$10],
      out_buffer_ptr^[$140+i*$14+$04], 13); {Parameters.}
  end;

{Unpack patterns. Align 1st pattern on paragraph boundary in output.}
  noc:= -1;
  o:= ((($140+noi*$14) shr 4)+1) shl 4;
  out_buffer_ptr^[$3e]:= (o shr 4) mod 256;
  out_buffer_ptr^[$3f]:= (o shr 4) div 256;
  for i:= 0 to nop-1 do
  begin
    p:= 16*(in_buffer_ptr^[$60+in_buffer_ptr^[$20]+
      in_buffer_ptr^[$22]*2+2*i]+
      in_buffer_ptr^[$60+in_buffer_ptr^[$20]+
      in_buffer_ptr^[$22]*2+2*i+1]*256)+2; {Pointer to pattern data.}
    fillchar(out_buffer_ptr^[o+i*64*_max_noc*5], 64*_max_noc*5, 255); {Clear.}
    for r:= 0 to 63 do
    begin
      repeat
        pack_info:= in_buffer_ptr^[p];
        inc(p);
        if (pack_info and $20)=$20 then
        begin
          if (pack_info and $1f)>noc then noc:=(pack_info and $1f);
          out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5+(pack_info and $1f)*5+0]:=
          in_buffer_ptr^[p];
          inc(p);
          out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5+(pack_info and $1f)*5+1]:=
          in_buffer_ptr^[p];
          inc(p);
        end;
        if (pack_info and $40)=$40 then
        begin
          if (pack_info and $1f)>noc then noc:=(pack_info and $1f);
          out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5+(pack_info and $1f)*5+2]:=
          in_buffer_ptr^[p];
          inc(p);
        end;
        if (pack_info and $80)=$80 then
        begin
          if (pack_info and $1f)>noc then noc:=(pack_info and $1f);
          out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5+(pack_info and $1f)*5+3]:=
          in_buffer_ptr^[p];
          inc(p);
          out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5+(pack_info and $1f)*5+4]:=
          in_buffer_ptr^[p];
          inc(p);
        end;
      until pack_info=0;
    end;
  end;
{Set number of channels.}
  inc(noc);
  out_buffer_ptr^[$24]:= noc mod 256;
  out_buffer_ptr^[$25]:= noc div 256;
{Write header.}
  blockwrite(out_file, out_buffer_ptr^[0], o);
{Write patterns row by row.}
  for i:= 0 to nop-1 do
  begin
    for r:= 0 to 63 do
      blockwrite(out_file, out_buffer_ptr^[o+i*64*_max_noc*5+r*_max_noc*5], noc*5);
  end;
  dispose(in_buffer_ptr);
  dispose(out_buffer_ptr);
end;

begin
  if paramcount=2 then
  begin
    source_file_name:= fexpand(paramstr(1));
    destination_file_name:= fexpand(paramstr(2));
    if source_file_name<>destination_file_name then
    begin
      assign(source_file, source_file_name);
      assign(destination_file, destination_file_name);
      reset(source_file, 1);
      if ioresult=0 then
      begin
        if filesize(source_file)>_buffer_size then
          writeln('WARNING: Source file is too big.');
        reset(destination_file, 1);
        if ioresult=0 then
        begin
          write('WARNING: Output file already exists. ');
          repeat
            write('Overwrite? [Y]es, [N]o: ');
            readln(answer);
          until (upcase(answer)='Y') or (upcase(answer)='N');
          really:= (upcase(answer)='Y');
        end
        else really:= true;
        if really then
        begin
          rewrite(destination_file, 1);
          Convert(source_file, destination_file);
        end
        else writeln('Operation cancelled.');
        close(destination_file);
      end
      else writeln('ERROR: Could not open source file.');
      close(source_file);
    end
    else writeln('ERROR: Source and destination files must be different.');
  end
  else Write_help;
end.
