{$I-}

uses dos;

const
  _datas_per_line = 8;
  _tab = #9;

var
  destination_file_name, command_string: string;
  destination_file: text;
  command, error_code: integer;
  datas_num: longint;
  answer: char;
  really, error: boolean;
  minimum, maximum, minimum_step: real;
  angle_start, angle_end: real;

procedure Display_help;
begin
  writeln('Sine data include file generator by Zed/reMal.');
  writeln('This program can generate a sine include file.');
  writeln('Parameters:');
  writeln('SINEDAT <str> <str> <num> <num> <num> <num> <num> <num>');
  writeln('                                             ');
  writeln(' command                                   ');
  writeln(' output file                              ');
  writeln(' # of entries                         ');
  writeln(' minimum                    ');
  writeln(' maximum               ');
  writeln(' minimum difference          ');
  writeln(' start angle     ');
  writeln(' end angle');
  writeln('    Commands are:');
  writeln(' -ab: Assembly-style output file of bytes.');
  writeln(' -aw: Assembly-style output file of words.');
  writeln(' -ad: Assembly-style output file of doublewords.');
  writeln(' -pb: Turbo Pascal-style output file of bytes.');
  writeln(' -pw: Turbo Pascal-style output file of words.');
  writeln(' -pd: Turbo Pascal-style output file of doublewords.');
end;

function Sine(angle, sine_size: real): real;

begin
  sine:= sin(angle/180*pi)*sine_size;
end;

function Make_sine_dat(command: integer; var destination_file: text;
  datas_num: longint; minimum, maximum, minimum_step,
  angle_start, angle_end: real): integer;
var
  i: longint;
  sine_size, center, angle, angle_step: real;
  temp_str, line_str, line_begin, line_end, directive_str: string;
  field_length: integer;
  error: boolean;

begin
  error:= false;
  case command and $0f of
    1:
    begin
      field_length:= 4;
      directive_str:= 'db';
    end;
    2:
    begin
      field_length:= 6;
      directive_str:= 'dw';
    end;
    3:
    begin
      field_length:= 11;
      directive_str:= 'dd';
    end;
    else error:= true;
  end;
  case command shr 4 of
    0:
    begin
      line_begin:= _tab+directive_str+_tab;
      line_end:= '';
    end;
    1:
    begin
      line_begin:= '  ';
      line_end:= ',';
    end;
    else error:= true;
  end;

  if not(error) then
  begin
    sine_size:= (maximum-minimum)/2;
    center:= (maximum+minimum)/2;
    angle_step := (angle_end-angle_start)/datas_num;

    angle:= angle_start;
    for i:= 1 to datas_num do
    begin
      str(round(round((center+sine(angle, sine_size))/minimum_step)*minimum_step): field_length, temp_str);
      case (i-1) mod _datas_per_line of
        0: write(destination_file, line_begin, temp_str);
        _datas_per_line-1: writeln(destination_file, ','+temp_str+line_end);
        else write(destination_file, ','+temp_str);
      end;
      angle:= angle+angle_step;
    end;
  end;
  Make_sine_dat:= 0;
end;

begin
  case paramcount of
    8:
    begin
      command_string:= paramstr(1);
      destination_file_name:= fexpand(paramstr(2));
      error:= false;
      command:= 0;
      if command_string='-ab' then command:= $01;
      if command_string='-aw' then command:= $02;
      if command_string='-ad' then command:= $03;
      if command_string='-pb' then command:= $11;
      if command_string='-pw' then command:= $12;
      if command_string='-pd' then command:= $13;
      val(paramstr(3), datas_num, error_code);
      if error_code<>0 then error:= error or true;
      val(paramstr(4), minimum, error_code);
      if error_code<>0 then error:= error or true;
      val(paramstr(5), maximum, error_code);
      if error_code<>0 then error:= error or true;
      val(paramstr(6), minimum_step, error_code);
      if error_code<>0 then error:= error or true;
      val(paramstr(7), angle_start, error_code);
      if error_code<>0 then error:= error or true;
      val(paramstr(8), angle_end, error_code);
      if error_code<>0 then error:= error or true;
      if (command<>0) and not(error) then
      begin
        assign(destination_file, destination_file_name);
        reset(destination_file);
        if ioresult=0 then
        begin
          write('WARNING: Output file already exists. ');
          repeat
            write('Overwrite? [Y]es, [N]o: ');
            readln(answer);
          until (upcase(answer)='Y') or (upcase(answer)='N');
          really:= (upcase(answer)='Y');
        end
        else really:= true;
        if really then
        begin
          rewrite(destination_file);
          if Make_sine_dat(command, destination_file, datas_num, minimum,
            maximum, minimum_step, angle_start, angle_end)<>0 then
            writeln('ERROR: Data generation failed!');
        end
        else writeln('ERROR: Operation cancelled.');
        close(destination_file);
      end
      else Display_help;
    end
    else Display_help;
  end;
end.
